"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


AuditLogAction = Union[
    Literal["created", "updated", "deleted", "voided", "captured"], UnrecognizedStr
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class GiftCardTransactionCreateTypedDict(TypedDict):
    r"""Create a charge against a gift card"""

    number: str
    r"""The 16-19 digit number for the gift card."""
    pin: str
    r"""The PIN for this gift card."""
    amount: int
    r"""The monetary amount for this transaction to charge against the gift card, in the smallest currency unit (for example, cents or pence)."""


class GiftCardTransactionCreate(BaseModel):
    r"""Create a charge against a gift card"""

    number: str
    r"""The 16-19 digit number for the gift card."""

    pin: str
    r"""The PIN for this gift card."""

    amount: int
    r"""The monetary amount for this transaction to charge against the gift card, in the smallest currency unit (for example, cents or pence)."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, NotRequired, TypedDict


class ClickToPayPaymentMethodCreateTypedDict(TypedDict):
    r"""Create a Click to Pay payment with a decrypted token and cryptogram. This
    is mainly used internally but can be used by anyone with their own C2P
    integration.
    """

    token: str
    r"""The device token."""
    cryptogram: str
    r"""The payment cryptogram for the device token."""
    expiration_date: str
    r"""The expiration date of the device token."""
    method: Literal["click-to-pay"]
    r"""Aways `click-to-pay`."""
    buyer_id: NotRequired[Nullable[str]]
    r"""The ID of the buyer to associate this transaction to."""
    buyer_external_identifier: NotRequired[Nullable[str]]
    r"""The external identifier of the buyer to create a transaction for."""
    external_identifier: NotRequired[Nullable[str]]
    r"""The external identifier of the payment method to filter by."""
    redirect_url: NotRequired[Nullable[str]]
    r"""The URL to redirect a user back to after the complete 3DS in browser."""


class ClickToPayPaymentMethodCreate(BaseModel):
    r"""Create a Click to Pay payment with a decrypted token and cryptogram. This
    is mainly used internally but can be used by anyone with their own C2P
    integration.
    """

    token: str
    r"""The device token."""

    cryptogram: str
    r"""The payment cryptogram for the device token."""

    expiration_date: str
    r"""The expiration date of the device token."""

    METHOD: Annotated[
        Annotated[
            Literal["click-to-pay"], AfterValidator(validate_const("click-to-pay"))
        ],
        pydantic.Field(alias="method"),
    ] = "click-to-pay"
    r"""Aways `click-to-pay`."""

    buyer_id: OptionalNullable[str] = UNSET
    r"""The ID of the buyer to associate this transaction to."""

    buyer_external_identifier: OptionalNullable[str] = UNSET
    r"""The external identifier of the buyer to create a transaction for."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""The external identifier of the payment method to filter by."""

    redirect_url: OptionalNullable[str] = UNSET
    r"""The URL to redirect a user back to after the complete 3DS in browser."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "buyer_id",
            "buyer_external_identifier",
            "external_identifier",
            "redirect_url",
        ]
        nullable_fields = [
            "buyer_id",
            "buyer_external_identifier",
            "external_identifier",
            "redirect_url",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardscheme import CardScheme
from .checkoutsessionpaymentmethoddetails import (
    CheckoutSessionPaymentMethodDetails,
    CheckoutSessionPaymentMethodDetailsTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CheckoutSessionPaymentMethodTypedDict(TypedDict):
    type: Literal["payment-method"]
    r"""Always `payment-method`"""
    id: NotRequired[Nullable[str]]
    r"""The ID of the payment method."""
    details: NotRequired[Nullable[CheckoutSessionPaymentMethodDetailsTypedDict]]
    r"""Details for credit or debit card payment method."""
    label: NotRequired[Nullable[str]]
    r"""The last 4 digits of the the card."""
    method: Literal["card"]
    r"""Always `card`"""
    scheme: NotRequired[Nullable[CardScheme]]
    r"""The scheme of the card."""
    fingerprint: NotRequired[Nullable[str]]
    r"""The unique hash derived from the card number."""


class CheckoutSessionPaymentMethod(BaseModel):
    TYPE: Annotated[
        Annotated[
            Optional[Literal["payment-method"]],
            AfterValidator(validate_const("payment-method")),
        ],
        pydantic.Field(alias="type"),
    ] = "payment-method"
    r"""Always `payment-method`"""

    id: OptionalNullable[str] = UNSET
    r"""The ID of the payment method."""

    details: OptionalNullable[CheckoutSessionPaymentMethodDetails] = UNSET
    r"""Details for credit or debit card payment method."""

    label: OptionalNullable[str] = UNSET
    r"""The last 4 digits of the the card."""

    METHOD: Annotated[
        Annotated[Optional[Literal["card"]], AfterValidator(validate_const("card"))],
        pydantic.Field(alias="method"),
    ] = "card"
    r"""Always `card`"""

    scheme: Annotated[
        OptionalNullable[CardScheme], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The scheme of the card."""

    fingerprint: OptionalNullable[str] = UNSET
    r"""The unique hash derived from the card number."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "id",
            "details",
            "label",
            "method",
            "scheme",
            "fingerprint",
        ]
        nullable_fields = ["id", "details", "label", "scheme", "fingerprint"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from gr4vy.types import BaseModel, UnrecognizedStr
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Any, Dict, Literal, Optional, Union
from typing_extensions import Annotated, TypedDict


Name = Union[
    Literal[
        "transaction-updated-status",
        "transaction-sync-event",
        "transaction-sync-failed-event",
        "transaction-api-request",
        "transaction-api-response",
        "bin-lookup-request",
        "three-d-secure-success",
        "three-d-secure-request-error",
        "three-d-secure-preparation-request",
        "three-d-secure-authentication-request",
        "three-d-secure-result-request",
        "anti-fraud-decision",
        "anti-fraud-decision-error",
        "anti-fraud-decision-skipped",
        "anti-fraud-webhook",
        "anti-fraud-transaction-status-update",
        "anti-fraud-transaction-status-update-error",
        "anti-fraud-decision-update",
        "anti-fraud-decision-update-error",
        "gift-card-redemption-succeeded",
        "gift-card-redemption-failed",
        "gift-card-refund-succeeded",
        "gift-card-refund-failed",
        "gift-card-reversal-succeeded",
        "payment-connector-response-transaction-authorization-succeeded",
        "payment-connector-response-transaction-capture-succeeded",
        "payment-connector-response-transaction-authorization-failed",
        "payment-connector-response-transaction-declined",
        "payment-connector-response-transaction-capture-failed",
        "payment-connector-response-transaction-capture-declined",
        "payment-connector-response-transaction-void-succeeded",
        "payment-connector-response-transaction-void-declined",
        "payment-connector-response-transaction-void-failed",
        "payment-connector-external-transaction-request",
        "payment-connector-report-transaction-settled",
        "digital-wallet-apple-pay-token-decrypted",
        "digital-wallet-google-pay-token-decrypted",
        "digital-wallet-click-to-pay-token-decrypted",
        "network-token-provision-succeeded",
        "network-token-provision-failed",
        "network-token-cryptogram-provision-succeeded",
        "network-token-cryptogram-provision-failed",
        "the-giving-block-transaction-conversion-succeeded",
        "real-time-account-update",
    ],
    UnrecognizedStr,
]
r"""The specific event name."""


class TransactionEventTypedDict(TypedDict):
    id: str
    r"""The ID for the event."""
    name: Name
    r"""The specific event name."""
    created_at: datetime
    r"""The date this event was created at."""
    context: Dict[str, Any]
    type: Literal["transaction-event"]
    r"""Always `transaction-event`."""


class TransactionEvent(BaseModel):
    id: str
    r"""The ID for the event."""

    name: Annotated[Name, PlainValidator(validate_open_enum(False))]
    r"""The specific event name."""

    created_at: datetime
    r"""The date this event was created at."""

    context: Dict[str, Any]

    TYPE: Annotated[
        Annotated[
            Optional[Literal["transaction-event"]],
            AfterValidator(validate_const("transaction-event")),
        ],
        pydantic.Field(alias="type"),
    ] = "transaction-event"
    r"""Always `transaction-event`."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from ._version import (
    __gen_version__,
    __openapi_doc_version__,
    __user_agent__,
    __version__,
)
from .httpclient import AsyncHttpClient, HttpClient
from .utils import Logger, RetryConfig, remove_suffix
from dataclasses import dataclass, field
from gr4vy import models
from gr4vy.models import internal
from gr4vy.types import OptionalNullable, UNSET
from pydantic import Field
from typing import Callable, Dict, Optional, Tuple, Union


SERVER_PRODUCTION = "production"
SERVER_SANDBOX = "sandbox"
SERVERS = {
    SERVER_PRODUCTION: "https://api.{id}.gr4vy.app",
    SERVER_SANDBOX: "https://api.sandbox.{id}.gr4vy.app",
}
"""Contains the list of servers available to the SDK"""


@dataclass
class SDKConfiguration:
    client: Union[HttpClient, None]
    client_supplied: bool
    async_client: Union[AsyncHttpClient, None]
    async_client_supplied: bool
    debug_logger: Logger
    globals: internal.Globals
    security: Optional[Union[models.Security, Callable[[], models.Security]]] = None
    server_url: Optional[str] = ""
    server: Optional[str] = ""
    server_defaults: Dict[str, Dict[str, str]] = field(default_factory=Dict)
    language: str = "python"
    openapi_doc_version: str = __openapi_doc_version__
    sdk_version: str = __version__
    gen_version: str = __gen_version__
    user_agent: str = __user_agent__
    retry_config: OptionalNullable[RetryConfig] = Field(default_factory=lambda: UNSET)
    timeout_ms: Optional[int] = None

    def get_server_details(self) -> Tuple[str, Dict[str, str]]:
        if self.server_url is not None and self.server_url:
            return remove_suffix(self.server_url, "/"), {}
        if not self.server:
            self.server = SERVER_PRODUCTION

        if self.server not in SERVERS:
            raise ValueError(f'Invalid server "{self.server}"')

        return SERVERS[self.server], self.server_defaults.get(self.server, {})
